/*
 * xmms-kdedb.h               06.06.2002
 *
 */

#ifndef __XMMS_KDEDB_H__
#define __XMMS_KDEDB_H__

#include <string>

#if defined __GNUC__ && (__GNUC__ > 2)
  #include <ext/hash_set>
  using namespace __gnu_cxx;
#else
  #include <hash_set>
#endif

#include <kconfig.h>

#include <qhbuttongroup.h>
#include <qevent.h>
#include <qprogressbar.h>
#include <qfiledialog.h>
#include <qwidget.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qpoint.h>
#include <qpixmap.h>
#include <qsize.h>
#include <qradiobutton.h>
#include <qthread.h>
#include <qvbuttongroup.h>

#include "sqlite.h"

#include "playerinterface.h"

using namespace std;

class StatusLabel;
class ProgressLabel;

class InsertThread;

class XmmsKdeDB : public QObject {

  Q_OBJECT

public:
  XmmsKdeDB(KConfig *conf, QPixmap *icon);

  ~XmmsKdeDB();

  bool connectDB();

  void disconnectDB();

  bool isConnectedDB();

  bool isEnabled() {
    return enable;
  }

  void disableDB() {
    enable = false;
    disconnectDB();
  }

  void enableDB() {
    enable = true;
  }

  sqlite *db;
  sqlite *querydb;

  QWidget* getConfigurationWidget(QWidget *parent);
  void writeConfig(); 

signals:
  
  void statusChanged(QString status);

public slots:

  void stopInsertThread();
  void updateDatabase();

private slots:

  void addPathToList();
  void removePathFromList();

  void configurationChanged();

private:

  void readConfig();

  bool enable; 
  bool connected;

  QString name; 

  KConfig *config;

  QCheckBox *enableBox;
  QListBox *pathListBox;
  QStringList pathList;
  QFrame *statusFrame;
  QPixmap *icon;
  InsertThread *insertThread;
};


class XmmsKdeDBQuery : public QVBox {

  Q_OBJECT

public:

  XmmsKdeDBQuery(XmmsKdeDB *datab, PlayerInterface *p, 
		 QPixmap *icon, KConfig *conf);

  ~XmmsKdeDBQuery();

  void setPlayer(PlayerInterface *p);
  void popup();

  QWidget* getConfigurationWidget(QWidget *parent);
  void writeConfig();

  int getPopup() {

    return pop;
  }

signals:
  
public slots:

private slots:

  void newQuery();

  void play(int index);
  
  void setPlayList();
  void addPlayList();

  void popupChanged(int index);

private:

  void readConfig();

  XmmsKdeDB *db;
  PlayerInterface *player;

  QListBox *resultBox;
  QLineEdit *query;

  QHButtonGroup *buttons;

  KConfig *config;

  QVButtonGroup *queryGroup;
  QPoint framePos;
  QSize frameSize;
  int pop;
};

class QueryItem : public QListBoxText {


public:
  QueryItem(QString text, QString file);
  ~QueryItem();

  QString getFile();


private:

  QString filename;

};

class InsertThread : public QThread {

 public:
  InsertThread(sqlite *datab, QStringList dirs, StatusLabel *status,
	       StatusLabel *file, ProgressLabel *progress);
  virtual void run();

 private:
  void updateDatabase(QStringList dir);
  void insertIntoDatabase(QString file);
  void deleteFromDatabase(QString file);

  void addPathToVector(QString path, vector<QString *> *v);

  sqlite *db;
  QStringList dir;

  StatusLabel *statusLabel;
  StatusLabel *fileLabel;
  ProgressLabel *progressLabel;
};

class StatusEvent: public QCustomEvent {

 public:
  StatusEvent(QString m): QCustomEvent(60042), s(m) {}

  QString message() const {
    return s;
  }
 private:
  QString s;
};

class ProgressStepEvent: public QCustomEvent {

 public:
  ProgressStepEvent(int p): QCustomEvent(60043) {
    prog = p;
  }

  int progress() const {
    return prog;
  }
 private:
  int prog;
};

class ProgressTotalEvent: public QCustomEvent {

 public:
  ProgressTotalEvent(int p): QCustomEvent(60044) {
    prog = p;
  }

  int progress() const {
    return prog;
  }
 private:
  int prog;
};

class StatusLabel: public QLabel {

 public:
  StatusLabel(char *c, QWidget *parent, int len): QLabel(c, parent) {
    length = len;
  }

  void customEvent(QCustomEvent *e) {
    if (e->type() == 60042) {
      StatusEvent *s = (StatusEvent *) e;
      QString st = s->message();
      st.truncate(length);
      setText(st);
    }
  }

 private:
  int length;

};

class ProgressLabel: public QProgressBar {

 public:
  ProgressLabel(QWidget *parent): QProgressBar(parent) {

  }

  void customEvent(QCustomEvent *e) {
    if (e->type() == 60043) {
      // progress step
      ProgressStepEvent *s = (ProgressStepEvent *) e;
      setProgress(s->progress());

    } else if (e->type() == 60044) {
      // progress total
      ProgressTotalEvent *s = (ProgressTotalEvent *) e;
      setTotalSteps(s->progress());
    }
  }

 private:
  int length;

};
#endif



